using namespace QtQuick::Controls;
/**
 * Kirigami Abstract windows
 */
class AbstractApplicationWindow : public QtQuick.Controls.ApplicationWindow {
public:

Q_PROPERTY(Item pageStack READ dummyGetter_pageStack_ignore)
};
